#!/usr/bin/env python3
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import os
import subprocess
import sys

def main():
    for lib_file in sys.argv[1:]:
        if os.path.isfile(lib_file):
            sign(lib_file)

def sign(lib_file):
    ld_lib_path = os.path.realpath(os.path.join(lib_file, '..'))
    bin_path = os.path.realpath(os.path.join(ld_lib_path, '../bin'))

    env = os.environ.copy()
    if sys.platform == 'win32':
        env['PATH'] = os.pathsep.join((env['PATH'], ld_lib_path))
    else:
        env['LD_LIBRARY_PATH'] = env['DYLD_LIBRARY_PATH'] = ld_lib_path

    dev_null = open(os.devnull, 'wb')
    subprocess.check_call([os.path.join(bin_path, 'shlibsign'), '-C', '-v', '-i', lib_file], env=env, stdout=dev_null, stderr=dev_null)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.

import sys
import tomllib


def main():
    with open(sys.argv[1], "rb") as f:
        data = tomllib.load(f)

    for key, value in data["libfuzzer"].items():
        print(f"-{key}={value}")


if __name__ == "__main__":
    main()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from taskgraph.transforms.base import TransformSequence

transforms = TransformSequence()


@transforms.add
def add_base_rev(config, tasks):
    """Inject NSS_BASE_REV into coverage tasks so they can generate
    diff coverage reports against the base revision."""
    for task in tasks:
        if task["name"] in ("test-coverage", "fuzz-coverage", "clang-tidy"):
            base_rev = config.params.get("base_rev", "")
            if base_rev:
                env = task["worker"].setdefault("env", {})
                env["NSS_BASE_REV"] = base_rev
        yield task



#!/usr/bin/env python3
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

"""
Generate NSS release email text based on version number.

Usage: python3 generate_release_email.py <version> <previous_version> [output_file]

Example:
  python3 generate_release_email.py 3.118 3.117
  python3 generate_release_email.py 3.118.1 3.118 release_email_3.118.1.txt
"""

import os
import re
import sys
from datetime import datetime
from subprocess import check_output


def exit_with_failure(message):
    """Exit the script with an error message."""
    print(f"ERROR: {message}", file=sys.stderr)
    sys.exit(1)


def version_string_to_underscore(version_string):
    """Convert version string like '3.118' to '3_118'."""
    return version_string.replace('.', '_')


def version_string_to_RTM_tag(version_string):
    """Convert version string like '3.118' to 'NSS_3_118_RTM'."""
    parts = version_string.split('.')
    return "NSS_" + "_".join(parts) + "_RTM"


def get_nspr_version():
    """Read the NSPR version from automation/release/nspr-version.txt."""
    nspr_version_file = "automation/release/nspr-version.txt"
    try:
        with open(nspr_version_file, 'r') as f:
            return f.readline().strip()
    except FileNotFoundError:
        exit_with_failure(f"Could not find {nspr_version_file}. Are you running from the NSS root directory?")


def get_changes_from_hg(current_tag, previous_tag):
    """Extract bug changes from Mercurial log between two tags."""
    try:
        # Get log entries between previous tag and current tag
        command = ["hg", "log", "-r", f"{previous_tag}:{current_tag}", "--template", "{desc|firstline}\\n"]
        log_output = check_output(command).decode('utf-8')
    except Exception as e:
        exit_with_failure(f"Failed to get hg log: {e}")

    # Extract bug numbers and descriptions
    bug_lines = []
    for line in reversed(log_output.split('\n')):
        if 'Bug' in line or 'bug' in line:
            line = line.strip()
            # Remove reviewer information
            line = line.split("r=")[0].strip()

            # Match patterns like "Bug 1234567 Something" and convert to "Bug 1234567 - Something"
            line = re.sub(r'(Bug\s+\d+)\s+([^-])', r'\1 - \2', line, flags=re.IGNORECASE)

            # Clean up punctuation
            if line:
                line = line.rstrip(',')

            # Add a full stop at the end if there isn't one
            if line and not line.endswith('.'):
                line = line + '.'

            if line and line not in bug_lines:
                bug_lines.append(line)

    return bug_lines


def generate_email_content(version, nspr_version, bug_lines, release_date):
    """Generate the email content for the release announcement."""
    version_underscore = version_string_to_underscore(version)
    changes_text = "\n\n".join([f"    {line}" for line in bug_lines])

    email_content = f"""Network Security Services (NSS) {version} was released on {release_date}.



The HG tag is NSS_{version_underscore}_RTM. This version of NSS requires NSPR {nspr_version} or newer. The latest version of NSPR is {nspr_version}.

NSS {version} source distributions are available on ftp.mozilla.org for secure HTTPS download:

<https://ftp.mozilla.org/pub/security/nss/releases/NSS_{version_underscore}_RTM/src/>

Changes:

{changes_text}

NSS {version} shared libraries are backwards-compatible with all older NSS 3.x shared libraries. A program linked with older NSS 3.x shared libraries will work with this new version of the shared libraries without recompiling or relinking. Furthermore, applications that restrict their use of NSS APIs to the functions listed in NSS Public Functions will remain compatible with future versions of the NSS shared libraries.

Bugs discovered should be reported by filing a bug report at <https://bugzilla.mozilla.org/enter_bug.cgi?product=NSS>

Release notes are available at <https://firefox-source-docs.mozilla.org/security/nss/releases/index.html>.
"""
    return email_content


def main():
    if len(sys.argv) < 3:
        print(__doc__)
        sys.exit(1)

    version = sys.argv[1].strip()
    previous_version = sys.argv[2].strip()

    # Determine output file (optional)
    output_file = None
    if len(sys.argv) >= 4:
        output_file = sys.argv[3].strip()

    # Get current date
    current_date = datetime.now().strftime("%-d %B %Y")

    # Get NSPR version
    nspr_version = get_nspr_version()

    # Convert versions to tags
    current_tag = version_string_to_RTM_tag(version)
    previous_tag = version_string_to_RTM_tag(previous_version)

    print(f"Generating release email for NSS {version}")
    print(f"Previous version: {previous_version}")
    print(f"Current tag: {current_tag}")
    print(f"Previous tag: {previous_tag}")
    print(f"NSPR version: {nspr_version}")
    print(f"Release date: {current_date}")
    print()

    # Get changes from Mercurial
    print("Extracting changes from Mercurial...")
    bug_lines = get_changes_from_hg(current_tag, previous_tag)
    print(f"Found {len(bug_lines)} bug entries")
    print()

    # Generate email content
    email_content = generate_email_content(version, nspr_version, bug_lines, current_date)

    # Write to file if specified, otherwise print to stdout
    if output_file:
        with open(output_file, 'w') as f:
            f.write(email_content)
        print(f"Release email written to: {output_file}")
        print()

    print("=" * 70)
    print("Email Content:")
    print("=" * 70)
    print(email_content)


if __name__ == "__main__":
    main()




# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from taskgraph.transforms.base import TransformSequence
from taskgraph.util.schema import resolve_keyed_by

transforms = TransformSequence()


@transforms.add
def resolve_platform_differences(config, tasks):
    for task in tasks:
        for key in ("worker-type", "run.command", "scopes", "worker.artifacts", "worker.taskcluster-proxy", "worker.max-run-time"):
            resolve_keyed_by(task, key, task["name"], platform=task["attributes"]["build_platform"])
        yield task


@transforms.add
def add_env_vars(config, tasks):
    for task in tasks:
        env = task["worker"].setdefault("env", {})
        if task["attributes"]["build_platform"].startswith(("mac", "windows")):
            env.update({"DOMSUF": "localdomain", "HOST": "localhost"})

        if task["attributes"]["build_platform"].startswith("mac"):
            env.update({"NSS_TASKCLUSTER_MAC": "1"})

        if task["attributes"].get("static"):
            env.update({"NSS_DISABLE_LIBPKIX": "1"})

        if config.params["try_options"].get("nspr_patch"):
            env.update({"ALLOW_NSPR_PATCH": "1"})

        yield task


@transforms.add
def remove_docker_image_for_gw(config, tasks):
    for task in tasks:
        if task["attributes"]["build_platform"].startswith(("mac", "windows")):
            task["worker"].pop("docker-image", None)
        yield task



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from taskgraph.transforms.base import TransformSequence
from taskgraph.util.set_name import set_name_strip_kind

transforms = TransformSequence()


@transforms.add
def set_task_name(config, tasks):
    # set the name of tasks created by the from_deps transforms by appending
    # the primary dependency's label to the name from kind.yml
    for task in tasks:
        primary_kind = task["attributes"]["primary-kind-dependency"]
        primary_dep = [dep for dep in config.kind_dependencies_tasks.values() if dep.label in task["dependencies"].values()][0]
        task["name"] += f"-{set_name_strip_kind(config, [], primary_dep, primary_kind)}"
        yield task



#!/usr/bin/env python3
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

"""
Generate NSS release documentation (RST file) based on version number.

Usage: python3 generate_release_doc.py <version> <previous_version> [output_file]

Example:
  python3 generate_release_doc.py 3.118 3.117
  python3 generate_release_doc.py 3.118.1 3.118 doc/rst/releases/nss_3_118_1.rst
"""

import os
import re
import sys
from datetime import datetime
from subprocess import call, check_call, check_output


def exit_with_failure(message):
    """Exit the script with an error message."""
    print(f"ERROR: {message}", file=sys.stderr)
    sys.exit(1)


def version_string_to_underscore(version_string):
    """Convert version string like '3.118' to '3_118'."""
    return version_string.replace('.', '_')


def version_string_to_RTM_tag(version_string):
    """Convert version string like '3.118' to 'NSS_3_118_RTM'."""
    parts = version_string.split('.')
    return "NSS_" + "_".join(parts) + "_RTM"


def get_nspr_version():
    """Read the NSPR version from automation/release/nspr-version.txt."""
    nspr_version_file = "automation/release/nspr-version.txt"
    try:
        with open(nspr_version_file, 'r') as f:
            return f.readline().strip()
    except FileNotFoundError:
        exit_with_failure(f"Could not find {nspr_version_file}. Are you running from the NSS root directory?")


def get_changes_from_hg(current_tag, previous_tag):
    """Extract bug changes from Mercurial log between two tags."""
    try:
        # Get log entries between previous tag and current tag
        command = ["hg", "log", "-r", f"{previous_tag}:{current_tag}", "--template", "{desc|firstline}\\n"]
        log_output = check_output(command).decode('utf-8')
    except Exception as e:
        exit_with_failure(f"Failed to get hg log: {e}")

    # Extract bug numbers and descriptions
    bug_lines = []
    for line in reversed(log_output.split('\n')):
        if 'Bug' in line or 'bug' in line:
            line = line.strip()
            # Remove reviewer information
            line = line.split("r=")[0].strip()

            # Match patterns like "Bug 1234567 Something" and convert to "Bug 1234567 - Something"
            line = re.sub(r'(Bug\s+\d+)\s+([^-])', r'\1 - \2', line, flags=re.IGNORECASE)

            # Clean up punctuation
            if line:
                line = line.rstrip(',')

            # Add a full stop at the end if there isn't one
            if line and not line.endswith('.'):
                line = line + '.'

            if line and line not in bug_lines:
                bug_lines.append(line)

    return bug_lines


def generate_rst_content(version, nspr_version, bug_lines, release_date):
    """Generate the RST content for the release notes."""
    version_underscore = version_string_to_underscore(version)
    changes_text = "\n".join([f"   - {line}" for line in bug_lines])

    rst_content = f""".. _mozilla_projects_nss_nss_{version_underscore}_release_notes:

NSS {version} release notes
{"=" * len(f"NSS {version} release notes")}

`Introduction <#introduction>`__
--------------------------------

.. container::

   Network Security Services (NSS) {version} was released on *{release_date}**.

`Distribution Information <#distribution_information>`__
--------------------------------------------------------

.. container::

   The HG tag is NSS_{version_underscore}_RTM. NSS {version} requires NSPR {nspr_version} or newer.

   NSS {version} source distributions are available on ftp.mozilla.org for secure HTTPS download:

   -  Source tarballs:
      https://ftp.mozilla.org/pub/mozilla.org/security/nss/releases/NSS_{version_underscore}_RTM/src/

   Other releases are available :ref:`mozilla_projects_nss_releases`.

.. _changes_in_nss_{version}:

`Changes in NSS {version} <#changes_in_nss_{version}>`__
------------------------------------------------------------------

.. container::

{changes_text}

"""
    return rst_content


def main():
    if len(sys.argv) < 3:
        print(__doc__)
        sys.exit(1)

    version = sys.argv[1].strip()
    previous_version = sys.argv[2].strip()

    # Determine output file
    if len(sys.argv) >= 4:
        output_file = sys.argv[3].strip()
    else:
        version_underscore = version_string_to_underscore(version)
        output_file = f"doc/rst/releases/nss_{version_underscore}.rst"

    # Get current date
    current_date = datetime.now().strftime("%-d %B %Y")

    # Get NSPR version
    nspr_version = get_nspr_version()

    # Convert versions to tags
    current_tag = version_string_to_RTM_tag(version)
    previous_tag = version_string_to_RTM_tag(previous_version)

    print(f"Generating release documentation for NSS {version}")
    print(f"Previous version: {previous_version}")
    print(f"Current tag: {current_tag}")
    print(f"Previous tag: {previous_tag}")
    print(f"NSPR version: {nspr_version}")
    print(f"Release date: {current_date}")
    print()

    # Get changes from Mercurial
    print("Extracting changes from Mercurial...")
    bug_lines = get_changes_from_hg(current_tag, previous_tag)
    print(f"Found {len(bug_lines)} bug entries")
    print()

    # Generate RST content
    rst_content = generate_rst_content(version, nspr_version, bug_lines, current_date)

    # Write to file
    os.makedirs(os.path.dirname(output_file), exist_ok=True)
    with open(output_file, 'w') as f:
        f.write(rst_content)

    print(f"Release documentation written to: {output_file}")
    print()
    print("Running doc-lint...")
    rc = call(["./mach", "doc-lint"])
    if rc != 0:
        exit_with_failure(f"doc-lint failed with exit status {rc}")
    print()
    print("=" * 70)
    print("Preview:")
    print("=" * 70)
    print(rst_content)


if __name__ == "__main__":
    main()




# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

project = "NSS"
extensions = []
root_doc = "index"

# Suppress cross-reference warnings for labels outside the NSS standalone
# build (e.g. :ref: to Firefox-managed docs).
suppress_warnings = ["ref.ref"]

exclude_patterns = []



#!/usr/bin/env python3
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

"""Run clang-tidy on NSS source files using compile_commands.json."""

import argparse
import json
import os
import re
import subprocess
import sys
from concurrent.futures import ThreadPoolExecutor, as_completed

EXTENSIONS = {".c", ".cc", ".cpp", ".cxx"}

DEFAULT_EXCLUDES = {
    "gtests/google_test",
    "lib/sqlite",
    "lib/zlib",
    "lib/freebl/verified",
}


def find_compile_commands(repo_root):
    """Locate compile_commands.json in common build output locations."""
    candidates = [
        os.path.join(repo_root, "out", "Debug", "compile_commands.json"),
        os.path.join(repo_root, "out", "Release", "compile_commands.json"),
    ]
    for path in candidates:
        if os.path.isfile(path):
            return path
    return None


def ensure_compile_commands(repo_root):
    """Build NSS and generate compile_commands.json if missing.

    Returns the path to compile_commands.json, or None on failure.
    """
    cc_path = find_compile_commands(repo_root)
    if cc_path:
        return cc_path

    target_dir = os.path.join(repo_root, "out", "Debug")
    cc_path = os.path.join(target_dir, "compile_commands.json")

    print("compile_commands.json not found — building NSS...", file=sys.stderr)
    result = subprocess.run([os.path.join(repo_root, "build.sh")], cwd=repo_root)
    if result.returncode != 0:
        print("error: build.sh failed", file=sys.stderr)
        return None

    print("Generating compile_commands.json...", file=sys.stderr)
    with open(cc_path, "w") as f:
        result = subprocess.run(
            ["ninja", "-C", target_dir, "-t", "compdb"], stdout=f, cwd=repo_root
        )
    if result.returncode != 0:
        print("error: ninja compdb failed", file=sys.stderr)
        return None

    return cc_path


def load_translation_units(
    compile_commands_path, repo_root, include_paths=None, exclude_paths=None, files=None
):
    """Load and filter translation units from compile_commands.json.

    Args:
        compile_commands_path: Path to compile_commands.json.
        repo_root: Repository root for computing relative paths.
        include_paths: If set, only include files under these prefixes.
        exclude_paths: Exclude files matching these prefixes.
        files: If set, only include these specific files.

    Returns:
        List of (file, directory, command) tuples.
    """
    with open(compile_commands_path) as f:
        entries = json.load(f)
    excludes = DEFAULT_EXCLUDES | set(exclude_paths or [])

    units = []
    seen = set()
    for entry in entries:
        src = entry.get("file", "")
        directory = entry.get("directory", "")
        # Resolve to repo-relative path for filtering.
        if os.path.isabs(src):
            abs_src = src
        else:
            abs_src = os.path.normpath(os.path.join(directory, src))
        rel = os.path.relpath(abs_src, repo_root)

        ext = os.path.splitext(rel)[1]
        if ext not in EXTENSIONS:
            continue

        if any(rel.startswith(ex) for ex in excludes):
            continue

        if include_paths and not any(rel.startswith(p) for p in include_paths):
            continue

        if files is not None:
            # Match against both absolute and relative paths.
            if rel not in files and abs_src not in files and src not in files:
                continue

        # Deduplicate: a file may appear multiple times in
        # compile_commands.json (e.g. compiled for different targets).
        # Only analyse each source file once.
        if rel in seen:
            continue
        seen.add(rel)

        units.append((src, directory, entry))

    return units


def parse_diff(diff_text):
    """Parse unified diff output into a dict of {file: [[start, end], ...]}.

    Only tracks added/modified line ranges (the ``+`` side of the diff) so
    that we can tell clang-tidy which lines are "interesting".
    """
    file_lines = {}
    current_file = None
    hunk_re = re.compile(r"^@@ -\d+(?:,\d+)? \+(\d+)(?:,(\d+))? @@")

    for line in diff_text.splitlines():
        # Detect the destination filename.
        if line.startswith("+++ b/"):
            current_file = line[6:]
            continue
        if line.startswith("+++ "):
            # hg-style: +++ b/path or +++ path
            current_file = line[4:].lstrip("b/")
            continue

        m = hunk_re.match(line)
        if m and current_file is not None:
            start = int(m.group(1))
            count = int(m.group(2)) if m.group(2) is not None else 1
            if count == 0:
                # Pure deletion hunk — no new lines.
                continue
            end = start + count - 1
            file_lines.setdefault(current_file, []).append([start, end])

    return file_lines


def build_line_filter(changed_lines, repo_root, compile_commands_path):
    """Build a clang-tidy --line-filter JSON string from *changed_lines*.

    *changed_lines* is the dict returned by :func:`parse_diff`.
    *compile_commands_path* is used to determine the build directory so
    that paths in the filter match the ``file`` entries that clang-tidy
    sees from compile_commands.json.
    Returns a JSON string suitable for ``--line-filter=...``, or *None*
    if there are no entries.
    """
    if not changed_lines:
        return None
    # clang-tidy matches --line-filter names against the "file" field in
    # compile_commands.json.  For NSS those are typically relative to the
    # build directory (e.g. "../../lib/foo/bar.c").  Construct the same
    # relative paths so the filter actually matches.
    build_dir = os.path.dirname(os.path.abspath(compile_commands_path))
    entries = []
    for path, ranges in changed_lines.items():
        abs_path = os.path.normpath(os.path.join(repo_root, path))
        rel_path = os.path.relpath(abs_path, build_dir)
        entries.append({"name": rel_path, "lines": ranges})
    return json.dumps(entries, separators=(",", ":"))


def get_diff(repo_root, base_rev=None):
    """Obtain a unified diff from the VCS in *repo_root*.

    If *base_rev* is given it is used as the base revision; otherwise the
    working-directory changes are diffed.
    """
    if os.path.isdir(os.path.join(repo_root, ".hg")):
        cmd = ["hg", "diff", "-U0"]
        if base_rev:
            cmd += ["--rev", base_rev]
    elif os.path.isdir(os.path.join(repo_root, ".git")):
        if base_rev:
            cmd = ["git", "diff", "-U0", base_rev]
        else:
            cmd = ["git", "diff", "-U0", "HEAD"]
    else:
        return None

    result = subprocess.run(cmd, capture_output=True, text=True, cwd=repo_root)
    if result.returncode != 0:
        return None
    return result.stdout


def run_one(
    clang_tidy, src, directory, entry, checks, fix, fail_on_warnings, line_filter=None
):
    """Run clang-tidy on a single translation unit."""
    cmd = [clang_tidy, "-p", directory]

    if checks:
        cmd += ["--checks=" + checks]

    if fix:
        cmd += ["--fix"]

    if fail_on_warnings:
        cmd += ["--warnings-as-errors=*"]

    if line_filter:
        cmd += ["--line-filter=" + line_filter]

    cmd.append(src)

    result = subprocess.run(cmd, capture_output=True, text=True, cwd=directory)
    return src, result


def main(argv=None):
    parser = argparse.ArgumentParser(description="Run clang-tidy on NSS source files")
    parser.add_argument(
        "-p",
        "--compile-commands",
        help="Path to compile_commands.json (auto-detected if omitted)",
    )
    parser.add_argument(
        "-j",
        "--jobs",
        type=int,
        default=os.cpu_count(),
        help="Number of parallel jobs (default: CPU count)",
    )
    parser.add_argument(
        "--clang-tidy", default="clang-tidy", help="Path to clang-tidy binary"
    )
    parser.add_argument(
        "--checks", help="Override checks (passed to clang-tidy --checks)"
    )
    parser.add_argument(
        "--include",
        action="append",
        default=[],
        help="Only analyse files under these path prefixes",
    )
    parser.add_argument(
        "--exclude",
        action="append",
        default=[],
        help="Exclude files under these path prefixes",
    )
    parser.add_argument(
        "--fail-on-warnings", action="store_true", help="Treat all warnings as errors"
    )
    parser.add_argument(
        "--fix", action="store_true", help="Apply suggested fixes in-place"
    )
    parser.add_argument(
        "--build",
        action="store_true",
        help="Build NSS and generate compile_commands.json if missing",
    )
    parser.add_argument(
        "--diff",
        action="store_true",
        help="Only report diagnostics on changed lines (uses VCS diff)",
    )
    parser.add_argument(
        "--diff-base",
        help="Base revision for --diff (default: working-directory changes)",
    )
    parser.add_argument(
        "files", nargs="*", help="Specific files to check (default: all matching files)"
    )

    args = parser.parse_args(argv)

    # Locate repo root (parent of automation/).
    repo_root = os.path.dirname(
        os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    )

    # Find compile_commands.json.
    if args.compile_commands:
        cc_path = args.compile_commands
    elif args.build:
        cc_path = ensure_compile_commands(repo_root)
    else:
        cc_path = find_compile_commands(repo_root)

    if not cc_path or not os.path.isfile(cc_path):
        print(
            "error: compile_commands.json not found.  "
            "Build first with: ./build.sh (or pass --build)",
            file=sys.stderr,
        )
        return 1

    # Build a line filter when --diff is requested, and restrict the
    # file set to only files that appear in the diff.
    line_filter = None
    diff_files = None
    if args.diff or args.diff_base:
        diff_text = get_diff(repo_root, args.diff_base)
        if diff_text is None:
            print("error: could not obtain diff from VCS", file=sys.stderr)
            return 1
        changed_lines = parse_diff(diff_text)
        line_filter = build_line_filter(changed_lines, repo_root, cc_path)
        diff_files = set(os.path.normpath(f) for f in changed_lines)

    include_paths = args.include or None
    file_set = set(os.path.normpath(f) for f in args.files) if args.files else None

    # When diffing, restrict to changed files (intersect with any
    # explicit file list if one was provided).
    if diff_files is not None:
        if file_set is not None:
            file_set = file_set & diff_files
        else:
            file_set = diff_files

    units = load_translation_units(
        cc_path,
        repo_root,
        include_paths=include_paths,
        exclude_paths=args.exclude,
        files=file_set,
    )

    if not units:
        if file_set:
            print("No matching translation units for the given files.", file=sys.stderr)
            return 0
        print("No translation units found.", file=sys.stderr)
        return 1

    print("Running clang-tidy on {} file(s)...".format(len(units)), file=sys.stderr)

    jobs = args.jobs

    failures = []
    with ThreadPoolExecutor(max_workers=jobs) as pool:
        futures = {
            pool.submit(
                run_one,
                args.clang_tidy,
                src,
                directory,
                entry,
                args.checks,
                args.fix,
                args.fail_on_warnings,
                line_filter,
            ): src
            for src, directory, entry in units
        }

        for future in as_completed(futures):
            src, result = future.result()
            # Print any output (diagnostics) from clang-tidy.
            if result.stdout:
                print(result.stdout, end="")
            if result.stderr:
                print(result.stderr, end="", file=sys.stderr)
            if result.returncode != 0:
                failures.append(src)

    if failures:
        print(
            "\nclang-tidy found issues in {} file(s):".format(len(failures)),
            file=sys.stderr,
        )
        for f in sorted(failures):
            print("  {}".format(f), file=sys.stderr)
        return 1

    print("clang-tidy: no issues found.", file=sys.stderr)
    return 0


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.

import random
import string

ECH_CONFIGS ="AEX+DQBBcQAgACDh4IuiuhhInUcKZx5uYcehlG9PQ1ZlzhvVZyjJl7dscQAEAAEAAQASY2xvdWRmbGFyZS1lY2guY29tAAA="


def main():
    # Use Encrypted Client Hello with the given Base64-encoded ECHConfigs.
    if random.randint(0, 1):
        print(f"-N {ECH_CONFIGS}")

    # Configure a TLS 1.3 External PSK with the given hex string for a key.
    if random.randint(0, 1):
        print(f"-z 0x{''.join(random.choices(string.hexdigits, k=16))}")

    # Enable the session ticket extension.
    if random.randint(0, 1):
        print("-u")

    # Enable the cert_status extension (OCSP stapling).
    if random.randint(0, 1):
        print("-T")

    # Enable the signed_certificate_timestamp extension.
    if random.randint(0, 1):
        print("-U")

    # Enable the delegated credentials extension.
    if random.randint(0, 1):
        print("-B")

    # Enable the extended master secret extension [RFC7627).
    if random.randint(0, 1):
        print("-G")

    # Allow 0-RTT data (TLS 1.3 only).
    if random.randint(0, 1):
        print("-Z")

    # Enable Encrypted Client Hello GREASEing with the given padding size (0-255).
    if random.randint(0, 1):
        print(f"-i {random.randint(0, 255)}")

    # Enable middlebox compatibility mode (TLS 1.3 only).
    if random.randint(0, 1):
        print("-e")

    if random.randint(0, 1):
        print("--enable-rfc8701-grease")

    if random.randint(0, 1):
        print("--enable-ch-extension-permutation")

    if random.randint(0, 1):
        print("--zlib-certificate-compression")


if __name__ =="__main__":
    main()
